//
// MAX7219 Control. Assumes two 8x8 LED Matrices
//
// http://www.amazon.co.uk/MAX7219-Matrix-Display-Module-Board/dp/B00NQB4ICG/ref=lh_ni_t?ie=UTF8&psc=1&smid=A50DZI580G3JX
//
// http://playground.arduino.cc/Main/LedControl 
//
// D10 = Digital O/P CS Pin
// D11 = Digital O/P Clock Pin
// D12 = Digital O/P Data Pin
//
			

		
#include <avr/pgmspace.h>
#include <LedControl.h>

const byte inputPin = 7; // Pull this pin low for lights to work

const int numDevices = 6;      // number of MAX7219s used
const int dataPin = 12;
const int clkPin = 11;
const int csPin = 10;
LedControl lc=LedControl(dataPin,clkPin,csPin,numDevices);

#define DELAY_US             1000   // Delay time in MicroSeconds
unsigned long CurrentTime  = 0;     // this variable will be overwritten by micros() each iteration of loop
unsigned long PastTime     = 0;     // A record of the time before this read
unsigned long ElapsedTime  = 0;     // Elapsed time in uS

#define MAX_INT 32767
#define EMARKER ,{MAX_INT,MAX_INT,MAX_INT}

#define SCREEN1  0
#define SCREEN2  2
#define SCREEN3  4
#define SCREEN4  6
#define SCREEN5  8
#define SCREEN6  10
#define SCREEN7  12
#define SCREEN8  14
// etc...


// Drop this code into Arduino development environment		
		
#define LeftEye1 0		
#define RightEye1 1		
#define LeftEye2 2		
#define RightEye2 3		
#define LeftEye3 4		
#define RightEye3 5		
#define LeftEye4 6		
#define RightEye4 7		
#define LeftEye5 8		
#define RightEye5 9		
#define LeftEye6 10		
#define RightEye6 11		
#define LeftEye7 12		
#define RightEye7 13		
#define LeftEye8 14		
#define RightEye8 15		
#define LeftEye9 16		
#define RightEye9 17		
#define LeftEye10 18		
#define RightEye10 19		
#define LeftEye11 20		
#define RightEye11 21		
#define LeftEye12 22		
#define RightEye12 23		
#define LeftEye13 24		
#define RightEye13 25		
#define LeftEye14 26		
#define RightEye14 27		
#define LeftEye15 28		
#define RightEye15 29		
#define LeftEye16 30		
#define RightEye16 31		
#define LeftEye17 32		
#define RightEye17 33		
#define LeftEye18 34		
#define RightEye18 35		
#define LeftEye19 36		
#define RightEye19 37		
#define LeftEye20 38		
#define RightEye20 39		
#define LeftEye21 40		
#define RightEye21 41		
#define LeftEye22 42		
#define RightEye22 43		
#define LeftEye23 44		
#define RightEye23 45		
#define LeftEye24 46		
#define RightEye24 47		
#define LeftEye25 48		
#define RightEye25 49		
#define LeftEye26 50		
#define RightEye26 51		
#define LeftEye27 52		
#define RightEye27 53		
#define LeftEye28 54		
#define RightEye28 55		
		
typedef struct {		
	prog_uchar array1[8];	
} binaryArrayType;		
		
binaryArrayType binaryArray[56] =		
{		
	{ // LeftEye1, 0	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye1, 1	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye2, 2	
		B00000000,
		B00111100,
		B01000010,
		B01011010,
		B01011010,
		B01000010,
		B00111100,
		B00000000
	},	
	{ // RightEye2, 3	
		B00000000,
		B00111100,
		B01000010,
		B01011010,
		B01011010,
		B01000010,
		B00111100,
		B00000000
	},	
	{ // LeftEye3, 4	
		B00000000,
		B00111100,
		B00100100,
		B00110100,
		B00110100,
		B00100100,
		B00111100,
		B00000000
	},	
	{ // RightEye3, 5	
		B00000000,
		B00111100,
		B00100100,
		B00110100,
		B00110100,
		B00100100,
		B00111100,
		B00000000
	},	
	{ // LeftEye4, 6	
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00000000
	},	
	{ // RightEye4, 7	
		B00000000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000
	},	
	{ // LeftEye5, 8	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000010,
		B01111100
	},	
	{ // RightEye5, 9	
		B01111100,
		B10000010,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye6, 10	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000010,
		B10000100,
		B01111000
	},	
	{ // RightEye6, 11	
		B01111000,
		B10000100,
		B10000010,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye7, 12	
		B01111110,
		B11000001,
		B10000001,
		B10011001,
		B10011010,
		B10000100,
		B10001000,
		B01110000
	},	
	{ // RightEye7, 13	
		B01110000,
		B10001000,
		B10000100,
		B10011010,
		B10011001,
		B10000001,
		B11000001,
		B01111110
	},	
	{ // LeftEye8, 14	
		B00111110,
		B01000001,
		B10000001,
		B10011001,
		B10011010,
		B10000100,
		B01001000,
		B00110000
	},	
	{ // RightEye8, 15	
		B00110000,
		B01001000,
		B10000100,
		B10011010,
		B10011001,
		B10000001,
		B01000001,
		B00111110
	},	
	{ // LeftEye9, 16	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye9, 17	
		B01111110,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye10, 18	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B01111110
	},	
	{ // RightEye10, 19	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B01111110
	},	
	{ // LeftEye11, 20	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B01111110
	},	
	{ // RightEye11, 21	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B01111110
	},	
	{ // LeftEye12, 22	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B01111110
	},	
	{ // RightEye12, 23	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B10011001,
		B01111110
	},	
	{ // LeftEye13, 24	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B01111110
	},	
	{ // RightEye13, 25	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10011001,
		B01111110
	},	
	{ // LeftEye14, 26	
		B00000000,
		B00111100,
		B01000010,
		B01000010,
		B01000010,
		B01011010,
		B00111100,
		B00000000
	},	
	{ // RightEye14, 27	
		B00000000,
		B00111100,
		B01000010,
		B01000010,
		B01000010,
		B01011010,
		B00111100,
		B00000000
	},	
	{ // LeftEye15, 28	
		B00000000,
		B00111100,
		B00100100,
		B00100100,
		B00100100,
		B00111100,
		B00111100,
		B00000000
	},	
	{ // RightEye15, 29	
		B00000000,
		B00111100,
		B00100100,
		B00100100,
		B00100100,
		B00111100,
		B00111100,
		B00000000
	},	
	{ // LeftEye16, 30	
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00000000
	},	
	{ // RightEye16, 31	
		B00000000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000,
		B00011000
	},	
	{ // LeftEye17, 32	
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00000000
	},	
	{ // RightEye17, 33	
		B00000000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000,
		B00010000
	},	
	{ // LeftEye18, 34	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10001101,
		B01111110
	},	
	{ // RightEye18, 35	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10001101,
		B01111110
	},	
	{ // LeftEye19, 36	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000111,
		B01111110
	},	
	{ // RightEye19, 37	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000111,
		B01111110
	},	
	{ // LeftEye20, 38	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000011,
		B10000011,
		B01111110
	},	
	{ // RightEye20, 39	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000011,
		B10000011,
		B01111110
	},	
	{ // LeftEye21, 40	
		B00111110,
		B01000001,
		B10000001,
		B10011001,
		B10010010,
		B10000100,
		B01001000,
		B00110000
	},	
	{ // RightEye21, 41	
		B00110000,
		B01001000,
		B10000100,
		B10010010,
		B10011001,
		B10000001,
		B01000001,
		B00111110
	},	
	{ // LeftEye22, 42	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000011,
		B10000011,
		B10000001,
		B01111110
	},	
	{ // RightEye22, 43	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000011,
		B10000011,
		B10000001,
		B01111110
	},	
	{ // LeftEye23, 44	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000111,
		B10000111,
		B10000001,
		B01111110
	},	
	{ // RightEye23, 45	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000111,
		B10000111,
		B10000001,
		B01111110
	},	
	{ // LeftEye24, 46	
		B01111110,
		B10000001,
		B10000001,
		B10001101,
		B10001101,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye24, 47	
		B01111110,
		B10000001,
		B10000001,
		B10001101,
		B10001101,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye25, 48	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10001101,
		B01111110
	},	
	{ // RightEye25, 49	
		B01111110,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10001101,
		B10001101,
		B01111110
	},	
	{ // LeftEye26, 50	
		B01111110,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye26, 51	
		B01111110,
		B10000001,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye27, 52	
		B01111110,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye27, 53	
		B01111110,
		B10011001,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // LeftEye28, 54	
		B01111110,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	},	
	{ // RightEye28, 55	
		B01111110,
		B10011001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B10000001,
		B01111110
	}	
};		
	
	
	

		
typedef struct myframeType {			
	int animationIndex;  // Index pointer into binaryArray signifying animation frame		
	int frameDelay;	     // Approx delay in MilliSeconds to hold display this animated frame
        int frameLuminance;  // 0 ... 15, The intensity of the led matrix for a given frame	
} frameType;
	

frameType movie1[] = 
{
// Blink
{LeftEye1,6000,1}, {LeftEye2,5,1}, {LeftEye3,10,1}, {LeftEye4,10,1},   {LeftEye17,100,1}, {LeftEye4,10,1}, {LeftEye3,10,1}, {LeftEye2,5,1},
// Full wide again
{LeftEye1,3000,2},
// Frown
{LeftEye5,5,3},    {LeftEye6,5,4}, {LeftEye7,5,5},  {LeftEye8,4000,11}, {LeftEye7,5,5},    {LeftEye6,5,4},   {LeftEye5,5,3}
EMARKER
};


frameType movie2[] = 
{
// Look Right
{LeftEye9,6000,1},  {LeftEye10,30,1},   {LeftEye11,30,1},   {LeftEye12,30,1},    {LeftEye13,1000,1},  
// Blink Right
{LeftEye14,100,1},  {LeftEye15,5,1},   {LeftEye16,10,1},   {LeftEye17,10,1},   {LeftEye16,10,1},   {LeftEye15,10,1},   {LeftEye14,5,1},
// Full wide eyes 
{LeftEye13,400,1},  {LeftEye12,200,1},    {LeftEye11,100,1},    {LeftEye10,100,1},    {LeftEye9,100,1},    {LeftEye26,100,1},    {LeftEye27,200,1},    {LeftEye28,400,1},    {LeftEye27,100,1},    {LeftEye26,200,1},      {LeftEye9,800,1},  
// Frown
{LeftEye5,5,3},    {LeftEye6,5,4}, {LeftEye7,5,5},  {LeftEye8,4000,11}, {LeftEye7,5,5},    {LeftEye6,5,4},   {LeftEye5,5,3}
EMARKER
};


frameType movie3[] = 
{
// Frown
{LeftEye1,4000,1}, {LeftEye5,5,3},    {LeftEye6,5,4}, {LeftEye7,5,5},  {LeftEye21,2000,11}, {LeftEye7,5,5},    {LeftEye6,5,4},   {LeftEye5,5,3}
EMARKER
};


typedef struct {
    int frameCountCurrent;   // This field must be zero 
    int frameDelayCurrent;   // This field must be zero  
    int movieLength;         // This field must be zero. Auto calculated at start up in 'loop()', requires 'EMARKER' to be last element in 'frameType' array
    int whichScreen;         // Which screen you wish to show this movie on.
    myframeType *movie;      // Pointer to movie of interest (ie. a collection of frames)
} frameAnimationDescriptor;


frameAnimationDescriptor nowShowing[] =
{
{0,0,0,SCREEN1, movie1},  
{0,0,0,SCREEN2, movie2},
{0,0,0,SCREEN3, movie3}
};


void handleFrameAnimation(int whichScreen, int movielength, struct myframeType *movie, int *frameCountCurrent, int *frameDelayCurrent)
{
   for (int i = 0; i < 8; i++)
   {
     lc.setRow(whichScreen,     i, binaryArray[movie[*frameCountCurrent].animationIndex    ].array1[i]); 
     lc.setRow(whichScreen + 1, i, binaryArray[movie[*frameCountCurrent].animationIndex + 1].array1[i]); 
   }          
   lc.setIntensity(whichScreen,     movie[*frameCountCurrent].frameLuminance);       
   lc.setIntensity(whichScreen + 1, movie[*frameCountCurrent].frameLuminance);       

  *frameDelayCurrent = movie[*frameCountCurrent].frameDelay;
  
   if ((*frameCountCurrent) < (movielength-1))
     (*frameCountCurrent)++;
   else
     *frameCountCurrent = 0;
}


void setup()
{
   pinMode(inputPin, INPUT);
   for (int x=0; x<numDevices; x++){
        lc.shutdown(x,false);       //The MAX72XX is in power-saving mode on startup
        lc.setIntensity(x,3);       // Set the brightness to default value
        lc.clearDisplay(x);         // and clear the display
    }
}



void loop()
{ 
  // Initialise
  int index1;
  for (int index = 0; index < (sizeof(nowShowing)/sizeof(frameAnimationDescriptor)); index++)
  {
      index1 = 0;
      while (nowShowing[index].movie[index1].animationIndex != MAX_INT) // calculate the number of frames in the movie and store is movieLength 
      {
          nowShowing[index].movieLength++;
          index1++;
      }
      nowShowing[index].frameCountCurrent = 0;
      handleFrameAnimation(nowShowing[index].whichScreen, nowShowing[index].movieLength, nowShowing[index].movie, &nowShowing[index].frameCountCurrent, &nowShowing[index].frameDelayCurrent);
  }    

  CurrentTime = micros();  
  while (true){

    if (digitalRead(inputPin))
    {
        for (int x=0; x<numDevices; x++)
        {
             lc.shutdown(x,false);       //The MAX72XX is in power-saving mode on startup
             lc.setIntensity(x,3);       // Set the brightness to default value
             lc.clearDisplay(x);         // and clear the display
        }

        for (int index = 0; index < (sizeof(nowShowing)/sizeof(frameAnimationDescriptor)); index++)
        {
            nowShowing[index].frameCountCurrent = 0;
        }        

        for (int x=0; x<numDevices; x++)
        {
             lc.shutdown(x,false);       //The MAX72XX is in power-saving mode on startup
             lc.clearDisplay(x);         // and clear the display
        }
        
        while (digitalRead(inputPin)){}
    }    
    
    ElapsedTime = CurrentTime - PastTime; // handles roll-over, if currentTime is small, and pastTime large, the result rolls over to a small positive value, the time that has passed
    if(ElapsedTime >= DELAY_US) PastTime = CurrentTime;
    while (ElapsedTime < DELAY_US)
    {
      CurrentTime = micros(); 
      ElapsedTime = CurrentTime - PastTime; // handles roll-over, if currentTime is small, and pastTime large, the result rolls over to a small positive value, the time that has passed
      if(ElapsedTime >= DELAY_US) PastTime = CurrentTime;
    }
    CurrentTime = micros(); 

    for (int index = 0; index < (sizeof(nowShowing)/sizeof(frameAnimationDescriptor)); index++)
    {
      nowShowing[index].frameDelayCurrent--;
      if (nowShowing[index].frameDelayCurrent == 0)
        handleFrameAnimation(nowShowing[index].whichScreen, nowShowing[index].movieLength, nowShowing[index].movie, &nowShowing[index].frameCountCurrent, &nowShowing[index].frameDelayCurrent);
    }
  }
}








